Package: CytoPipelineGUI
Title: GUI's for visualization of flow cytometry data analysis
        pipelines
Version: 1.8.0
Authors@R: 
    c(person(given = "Philippe",
             family = "Hauchamps",
             role = c("aut", "cre"),
             email = "philippe.hauchamps@uclouvain.be",
             comment = c(ORCID = "0000-0003-2865-1852")),
      person(given = "Laurent", family = "Gatto",
             email = "laurent.gatto@uclouvain.be",
             role = "aut",
             comment = c(ORCID = "0000-0002-1520-2268")),
      person(given = "Dan",
             family = "Lin",
             role = "ctb",
             email = "dan.8.lin@gsk.com"))
Description: This package is the companion of the `CytoPipeline` package.
 It provides GUI's (shiny apps) for the visualization of flow cytometry data
 analysis pipelines that are run with `CytoPipeline`.
 Two shiny applications are provided, i.e.
 an interactive flow frame assessment and comparison tool and
 an interactive scale transformations visualization and adjustment tool.
License: GPL-3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
BugReports: https://github.com/UCLouvain-CBIO/CytoPipelineGUI/issues
URL: https://uclouvain-cbio.github.io/CytoPipelineGUI
biocViews: FlowCytometry, Preprocessing, QualityControl, WorkflowStep,
        ImmunoOncology, Software, Visualization, GUI, ShinyApps
Collate: plots.R shiny-functions.R shiny-scaleTransform-module.R
        shiny.R CytoPipelineGUI-package.R
Depends: R (>= 4.4), CytoPipeline (>= 1.9.3)
Imports: shiny, plotly, ggplot2, flowCore
Suggests: testthat (>= 3.0.0), vdiffr, diffviewer, knitr, rmarkdown,
        BiocStyle, patchwork
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/CytoPipelineGUI
git_branch: RELEASE_3_22
git_last_commit: 9a3910e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:27:05 UTC; biocbuild
Author: Philippe Hauchamps [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2865-1852>),
  Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>),
  Dan Lin [ctb]
Maintainer: Philippe Hauchamps <philippe.hauchamps@uclouvain.be>
