# Generated by roxygen2: do not edit by hand

export(CytoPipelineCheckApp)
export(ScaleTransformApp)
export(plotDiffFlowFrame)
export(plotScaleTransformedChannel)
export(plotSelectedFlowFrame)
export(plotSelectedWorkflow)
import(CytoPipeline)
import(shiny)
